<?php

namespace BitGoblin\MCST\Minecraft;

class Server {

  // server root directory
  private string $rootDir;
  // server name
  private string $serverName;
  // server Minecraft version
  private string $serverVersion;
  // PID file path
  private string $pidFilePath;
  // server run status
  private bool $state = false;
  // server config properties
  private ServerConfig $properties;

  // class constructor
  public function __construct($dir) {
    $dirBits = explode('/', $dir);
    $this->rootDir = $dir;
    $this->serverName = $dirBits[count($dirBits) - 1];
    $this->pidFilePath = $this->rootDir . '/pid.txt';
    $this->properties = new ServerConfig($this->rootDir . '/server.properties');

    // get server version
    $versionFile = join('/', array($this->rootDir, 'current_version.txt'));
    if (file_exists($versionFile)) {
      $this->serverVersion = file($versionFile)[0];
    }

    // search for server PID file
    if (file_exists($this->pidFilePath)) {
      // get and search for PID
      $pid = trim($this->getPid());
      if (file_exists('/proc/' . $pid)) {
        // set server state to true since it is running
        $this->state = true;
      } else {
        // delete the PID file since it's no longer needed
        unlink($this->pidFilePath);
      }
    }
  }

  // start the server
  public function start(): void {
    $command = $this->rootDir . '/start.sh > /dev/null 2>&1 & echo $!'; 
    exec($command, $output);
    // use lsof to 
    $pidResult = exec('lsof ' . $this->rootDir . '/server*.jar | grep .jar');
    $pid = preg_split('/\s+/', $pidResult, -1, PREG_SPLIT_NO_EMPTY)[1];

    $pidFile = fopen($this->pidFilePath, 'w');
    fwrite($pidFile, $pid);
    fclose($pidFile);

    // set server state to true
    $this->state = true;
  }

  // stop the server
  public function stop(): void {
    $pid = $this->getPid();
    if ($pid != -1) {
      // kill the process gracefully
      exec('kill -15 ' . $pid);
      // set server state to false
      $this->state = false;
    }    
  }

  // get the server's PID
  private function getPid(): int {
    // search for server PID file
    if (file_exists($this->pidFilePath)) {
      // get and return server PID
      return trim(file($this->pidFilePath)[0]);
    }

    // return a 'false' result
    return -1;
  }

  // get property from server properties file
  public function getProperty(string $param) {
    return $this->properties->get($param);
  }

  // update property in server properties file
  public function updateProperty(string $paramName, $paramValue) {
    $this->properties->update($paramName, $paramValue);
  }

  // getters & setters
  public function getDirectory(): string {
    return $this->rootDir;
  }
  public function getName(): string {
    return $this->serverName;
  }
  public function getVersion(): string {
    return $this->serverVersion;
  }
  public function getState(): bool {
    return $this->state;
  }

}
